// Safe Links Cleaner
// Copyright 2021 David Byers <david.byers@liu.se>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

// Shared code



const safelinksPatterns = [
    {
	name: 'Microsoft Defender For Office 365 Safe Links',
	host: '*.safelinks.protection.outlook.com',
	path: '[?]url=([^&]+).*'
    },
    {
	name: 'Barracuda Link Protection',
	host: 'linkprotect.cudasvc.com',
	path: 'url[?]a=([^&]+).*'
    },
    {
	name: 'Proofpoint (v3)',
	host: 'urldefense.com',
	path: 'v3/_+([^_]+).*'
    }
    // {
    //	 name: 'Proofpoint (v2)',
    //	 host: 'urldefense.proofpoint.com',
    //	 path: 'v2/url[?]u=([^&]+).*',
    //	 proc: (string) => string.replace('_', '/')
    // }
];


/**
 * Compute a list of url patterns from the link patterns.
 * @returns {Array} - Array of patterns.
 */
function computeSafelinksMatchPatterns() {
    return safelinksPatterns.map(({host, path}) => '*://' + host + '/*');
}


/**
 * Escape a hostname pattern for use in a regexp.
 * @param {string} host - The host name pattern.
 * @returns {string} A suitable regexp.
 */
function escapeHostname(host) {
    return host.replace('.', '[.]').replace('*', '[^/]*')
}


/**
 * Compute a regexp that matches all mangled links.
 * @returns {RegExp} A regexp that matches all possible mangled links.
 */
function computeSafelinksRegexp() {
    return new RegExp('(?:' +
		      safelinksPatterns.map(
			  ({host, path}) =>
			      `(?:https?://${escapeHostname(host)}/${path})`).join('|') +
		      ')',
	'gi');
}


const safelinksRegexp = computeSafelinksRegexp()
